/*
 * #%L
 * OME-XML Java library for working with OME-XML metadata structures.
 * %%
 * Copyright (C) 2006 - 2016 Open Microscopy Environment:
 *   - Massachusetts Institute of Technology
 *   - National Institutes of Health
 *   - University of Dundee
 *   - Board of Regents of the University of Wisconsin-Madison
 *   - Glencoe Software, Inc.
 * %%
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of any organization.
 * #L%
 */

/*-----------------------------------------------------------------------------
 *
 * THIS IS AUTOMATICALLY GENERATED CODE.  DO NOT MODIFY.
 *
 *-----------------------------------------------------------------------------
 */

package ome.xml.model.enums.handlers;

import java.util.Hashtable;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ome.xml.model.enums.Enumeration;
import ome.xml.model.enums.EnumerationException;
import ome.xml.model.enums.UnitsElectricPotential;

import ome.xml.model.primitives.*;
import ome.units.quantity.ElectricPotential;
import ome.units.unit.Unit;
import ome.units.UNITS;


/**
 * Enumeration handler for UnitsElectricPotential.
 */
public class UnitsElectricPotentialEnumHandler implements IEnumerationHandler {

  // -- Constants --

  /** Logger for this class. */
  private static final Logger LOGGER =
    LoggerFactory.getLogger(UnitsElectricPotentialEnumHandler.class);

  // -- Fields --

  /** Every UnitsElectricPotential value must match one of these patterns. */
  private static final Hashtable<String, String> patterns = makePatterns();

  private static Hashtable<String, String> makePatterns() {
    Hashtable<String, String> p = new Hashtable<>();
    // BEGIN Schema enumeration mappings
    p.put("^\\s*YV\\s*", "YV");
    p.put("^\\s*ZV\\s*", "ZV");
    p.put("^\\s*EV\\s*", "EV");
    p.put("^\\s*PV\\s*", "PV");
    p.put("^\\s*TV\\s*", "TV");
    p.put("^\\s*GV\\s*", "GV");
    p.put("^\\s*MV\\s*", "MV");
    p.put("^\\s*kV\\s*", "kV");
    p.put("^\\s*hV\\s*", "hV");
    p.put("^\\s*daV\\s*", "daV");
    p.put("^\\s*V\\s*", "V");
    p.put("^\\s*dV\\s*", "dV");
    p.put("^\\s*cV\\s*", "cV");
    p.put("^\\s*mV\\s*", "mV");
    p.put("^\\s*µV\\s*", "µV");
    p.put("^\\s*nV\\s*", "nV");
    p.put("^\\s*pV\\s*", "pV");
    p.put("^\\s*fV\\s*", "fV");
    p.put("^\\s*aV\\s*", "aV");
    p.put("^\\s*zV\\s*", "zV");
    p.put("^\\s*yV\\s*", "yV");
    return p;
  }

  // -- IEnumerationHandler API methods --

  /* @see IEnumerationHandler#getEnumeration(String) */
  public Enumeration getEnumeration(String value)
    throws EnumerationException
  {
    if (value != null) {
      for (String pattern : patterns.keySet()) {
        // case sensitive compare
        if (value.matches(pattern)) {
          String v = patterns.get(pattern);
          return UnitsElectricPotential.fromString(v);
        }
      }
    }
    LOGGER.warn("Unknown {} value '{}' will be stored as \"Other\"",
      "UnitsElectricPotential", value);
    throw new EnumerationException(this.getClass().getName() +
      " could not find enumeration for " + value);
  }


  @SuppressWarnings("unused")
  public Enumeration getEnumeration(ElectricPotential inUnit)
    throws EnumerationException {
    return getEnumeration(inUnit.unit().getSymbol());
  }

  public static Unit<ElectricPotential> getBaseUnit(UnitsElectricPotential inModelUnit)
  {
    Unit<ElectricPotential> theResult;
    // begin: should be lookup in template
    theResult = UNITS.VOLT;
    if (UnitsElectricPotential.YOTTAVOLT.equals(inModelUnit))
    {
      theResult = UNITS.YOTTAVOLT;
    }
    if (UnitsElectricPotential.ZETTAVOLT.equals(inModelUnit))
    {
      theResult = UNITS.ZETTAVOLT;
    }
    if (UnitsElectricPotential.EXAVOLT.equals(inModelUnit))
    {
      theResult = UNITS.EXAVOLT;
    }
    if (UnitsElectricPotential.PETAVOLT.equals(inModelUnit))
    {
      theResult = UNITS.PETAVOLT;
    }
    if (UnitsElectricPotential.TERAVOLT.equals(inModelUnit))
    {
      theResult = UNITS.TERAVOLT;
    }
    if (UnitsElectricPotential.GIGAVOLT.equals(inModelUnit))
    {
      theResult = UNITS.GIGAVOLT;
    }
    if (UnitsElectricPotential.MEGAVOLT.equals(inModelUnit))
    {
      theResult = UNITS.MEGAVOLT;
    }
    if (UnitsElectricPotential.KILOVOLT.equals(inModelUnit))
    {
      theResult = UNITS.KILOVOLT;
    }
    if (UnitsElectricPotential.HECTOVOLT.equals(inModelUnit))
    {
      theResult = UNITS.HECTOVOLT;
    }
    if (UnitsElectricPotential.DECAVOLT.equals(inModelUnit))
    {
      theResult = UNITS.DECAVOLT;
    }
    // Skip default unit VOLT
    if (UnitsElectricPotential.DECIVOLT.equals(inModelUnit))
    {
      theResult = UNITS.DECIVOLT;
    }
    if (UnitsElectricPotential.CENTIVOLT.equals(inModelUnit))
    {
      theResult = UNITS.CENTIVOLT;
    }
    if (UnitsElectricPotential.MILLIVOLT.equals(inModelUnit))
    {
      theResult = UNITS.MILLIVOLT;
    }
    if (UnitsElectricPotential.MICROVOLT.equals(inModelUnit))
    {
      theResult = UNITS.MICROVOLT;
    }
    if (UnitsElectricPotential.NANOVOLT.equals(inModelUnit))
    {
      theResult = UNITS.NANOVOLT;
    }
    if (UnitsElectricPotential.PICOVOLT.equals(inModelUnit))
    {
      theResult = UNITS.PICOVOLT;
    }
    if (UnitsElectricPotential.FEMTOVOLT.equals(inModelUnit))
    {
      theResult = UNITS.FEMTOVOLT;
    }
    if (UnitsElectricPotential.ATTOVOLT.equals(inModelUnit))
    {
      theResult = UNITS.ATTOVOLT;
    }
    if (UnitsElectricPotential.ZEPTOVOLT.equals(inModelUnit))
    {
      theResult = UNITS.ZEPTOVOLT;
    }
    if (UnitsElectricPotential.YOCTOVOLT.equals(inModelUnit))
    {
      theResult = UNITS.YOCTOVOLT;
    }
    // end: should be lookup in template
    return theResult;
  }

  public static <T extends PrimitiveNumber> ElectricPotential getQuantity(T inValue, UnitsElectricPotential inModelUnit)
    throws EnumerationException
  {
    // "Positive" variants are listed before "NonNegative" variants because they extend the "NonNegative" variants and
    // would otherwise be ignored because the "NonNegative" test would be chosen for both "Positive" and "NonNegative"
    // types.  We also process Long before Integer for the same reason since the static analyser also found this led
    // to the Long codepath being picked for the Integer type, though it's not as clear why that is happening.  The
    // current order passes static analysis with all codepaths being correctly visited.
    if (inValue instanceof PercentFraction) {
      PercentFraction typedValue = (PercentFraction) inValue;
      return new ome.units.quantity.ElectricPotential(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof PositiveFloat) {
      PositiveFloat typedValue = (PositiveFloat) inValue;
      return new ome.units.quantity.ElectricPotential(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof NonNegativeFloat) {
      NonNegativeFloat typedValue = (NonNegativeFloat) inValue;
      return new ome.units.quantity.ElectricPotential(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof PositiveLong) {
      PositiveLong typedValue = (PositiveLong) inValue;
      return new ome.units.quantity.ElectricPotential(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof NonNegativeLong) {
      NonNegativeLong typedValue = (NonNegativeLong) inValue;
      return new ome.units.quantity.ElectricPotential(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof PositiveInteger) {
      PositiveInteger typedValue = (PositiveInteger) inValue;
      return new ome.units.quantity.ElectricPotential(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof NonNegativeInteger) {
      NonNegativeInteger typedValue = (NonNegativeInteger) inValue;
      return new ome.units.quantity.ElectricPotential(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    LOGGER.warn("Unknown type '{}' cannot be used to create a 'ElectricPotential' quantity",
      inValue.getClass().getName());
    throw new EnumerationException("UnitsElectricPotentialEnumHandler: type '"
      + inValue.getClass().getName() + "' cannot be used to create a quantity");
  }

  public static <T extends Number> ElectricPotential getQuantity(T inValue, UnitsElectricPotential inModelUnit)
    throws EnumerationException
  {
    if (inValue instanceof Double) {
      Double doubleValue = (Double) inValue;
      return new ome.units.quantity.ElectricPotential(doubleValue, getBaseUnit(inModelUnit));
    }
    if (inValue instanceof Integer) {
      Integer intValue = (Integer) inValue;
      return new ome.units.quantity.ElectricPotential(intValue, getBaseUnit(inModelUnit));
    }
    LOGGER.warn("Unknown type '{}' cannot be used to create a 'ElectricPotential' quantity",
      inValue.getClass().getName());
    throw new EnumerationException("UnitsElectricPotentialEnumHandler: type '"
      + inValue.getClass().getName() + "' cannot be used to create a quantity");
  }

  /* @see IEnumerationHandler#getEntity() */
  public Class<? extends Enumeration> getEntity() {
    return UnitsElectricPotential.class;
  }

}
