/*
 * #%L
 * OME-XML Java library for working with OME-XML metadata structures.
 * %%
 * Copyright (C) 2006 - 2016 Open Microscopy Environment:
 *   - Massachusetts Institute of Technology
 *   - National Institutes of Health
 *   - University of Dundee
 *   - Board of Regents of the University of Wisconsin-Madison
 *   - Glencoe Software, Inc.
 * %%
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of any organization.
 * #L%
 */

/*-----------------------------------------------------------------------------
 *
 * THIS IS AUTOMATICALLY GENERATED CODE.  DO NOT MODIFY.
 *
 *-----------------------------------------------------------------------------
 */

package ome.xml.model.enums.handlers;

import java.util.Hashtable;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ome.xml.model.enums.Enumeration;
import ome.xml.model.enums.EnumerationException;
import ome.xml.model.enums.UnitsTime;

import ome.xml.model.primitives.*;
import ome.units.quantity.Time;
import ome.units.unit.Unit;
import ome.units.UNITS;


/**
 * Enumeration handler for UnitsTime.
 */
public class UnitsTimeEnumHandler implements IEnumerationHandler {

  // -- Constants --

  /** Logger for this class. */
  private static final Logger LOGGER =
    LoggerFactory.getLogger(UnitsTimeEnumHandler.class);

  // -- Fields --

  /** Every UnitsTime value must match one of these patterns. */
  private static final Hashtable<String, String> patterns = makePatterns();

  private static Hashtable<String, String> makePatterns() {
    Hashtable<String, String> p = new Hashtable<>();
    // BEGIN Schema enumeration mappings
    p.put("^\\s*Ys\\s*", "Ys");
    p.put("^\\s*Zs\\s*", "Zs");
    p.put("^\\s*Es\\s*", "Es");
    p.put("^\\s*Ps\\s*", "Ps");
    p.put("^\\s*Ts\\s*", "Ts");
    p.put("^\\s*Gs\\s*", "Gs");
    p.put("^\\s*Ms\\s*", "Ms");
    p.put("^\\s*ks\\s*", "ks");
    p.put("^\\s*hs\\s*", "hs");
    p.put("^\\s*das\\s*", "das");
    p.put("^\\s*s\\s*", "s");
    p.put("^\\s*ds\\s*", "ds");
    p.put("^\\s*cs\\s*", "cs");
    p.put("^\\s*ms\\s*", "ms");
    p.put("^\\s*µs\\s*", "µs");
    p.put("^\\s*ns\\s*", "ns");
    p.put("^\\s*ps\\s*", "ps");
    p.put("^\\s*fs\\s*", "fs");
    p.put("^\\s*as\\s*", "as");
    p.put("^\\s*zs\\s*", "zs");
    p.put("^\\s*ys\\s*", "ys");
    p.put("^\\s*min\\s*", "min");
    p.put("^\\s*h\\s*", "h");
    p.put("^\\s*d\\s*", "d");
    return p;
  }

  // -- IEnumerationHandler API methods --

  /* @see IEnumerationHandler#getEnumeration(String) */
  public Enumeration getEnumeration(String value)
    throws EnumerationException
  {
    if (value != null) {
      for (String pattern : patterns.keySet()) {
        // case sensitive compare
        if (value.matches(pattern)) {
          String v = patterns.get(pattern);
          return UnitsTime.fromString(v);
        }
      }
    }
    LOGGER.warn("Unknown {} value '{}' will be stored as \"Other\"",
      "UnitsTime", value);
    throw new EnumerationException(this.getClass().getName() +
      " could not find enumeration for " + value);
  }


  @SuppressWarnings("unused")
  public Enumeration getEnumeration(Time inUnit)
    throws EnumerationException {
    return getEnumeration(inUnit.unit().getSymbol());
  }

  public static Unit<Time> getBaseUnit(UnitsTime inModelUnit)
  {
    Unit<Time> theResult;
    // begin: should be lookup in template
    theResult = UNITS.SECOND;
    if (UnitsTime.YOTTASECOND.equals(inModelUnit))
    {
      theResult = UNITS.YOTTASECOND;
    }
    if (UnitsTime.ZETTASECOND.equals(inModelUnit))
    {
      theResult = UNITS.ZETTASECOND;
    }
    if (UnitsTime.EXASECOND.equals(inModelUnit))
    {
      theResult = UNITS.EXASECOND;
    }
    if (UnitsTime.PETASECOND.equals(inModelUnit))
    {
      theResult = UNITS.PETASECOND;
    }
    if (UnitsTime.TERASECOND.equals(inModelUnit))
    {
      theResult = UNITS.TERASECOND;
    }
    if (UnitsTime.GIGASECOND.equals(inModelUnit))
    {
      theResult = UNITS.GIGASECOND;
    }
    if (UnitsTime.MEGASECOND.equals(inModelUnit))
    {
      theResult = UNITS.MEGASECOND;
    }
    if (UnitsTime.KILOSECOND.equals(inModelUnit))
    {
      theResult = UNITS.KILOSECOND;
    }
    if (UnitsTime.HECTOSECOND.equals(inModelUnit))
    {
      theResult = UNITS.HECTOSECOND;
    }
    if (UnitsTime.DECASECOND.equals(inModelUnit))
    {
      theResult = UNITS.DECASECOND;
    }
    // Skip default unit SECOND
    if (UnitsTime.DECISECOND.equals(inModelUnit))
    {
      theResult = UNITS.DECISECOND;
    }
    if (UnitsTime.CENTISECOND.equals(inModelUnit))
    {
      theResult = UNITS.CENTISECOND;
    }
    if (UnitsTime.MILLISECOND.equals(inModelUnit))
    {
      theResult = UNITS.MILLISECOND;
    }
    if (UnitsTime.MICROSECOND.equals(inModelUnit))
    {
      theResult = UNITS.MICROSECOND;
    }
    if (UnitsTime.NANOSECOND.equals(inModelUnit))
    {
      theResult = UNITS.NANOSECOND;
    }
    if (UnitsTime.PICOSECOND.equals(inModelUnit))
    {
      theResult = UNITS.PICOSECOND;
    }
    if (UnitsTime.FEMTOSECOND.equals(inModelUnit))
    {
      theResult = UNITS.FEMTOSECOND;
    }
    if (UnitsTime.ATTOSECOND.equals(inModelUnit))
    {
      theResult = UNITS.ATTOSECOND;
    }
    if (UnitsTime.ZEPTOSECOND.equals(inModelUnit))
    {
      theResult = UNITS.ZEPTOSECOND;
    }
    if (UnitsTime.YOCTOSECOND.equals(inModelUnit))
    {
      theResult = UNITS.YOCTOSECOND;
    }
    if (UnitsTime.MINUTE.equals(inModelUnit))
    {
      theResult = UNITS.MINUTE;
    }
    if (UnitsTime.HOUR.equals(inModelUnit))
    {
      theResult = UNITS.HOUR;
    }
    if (UnitsTime.DAY.equals(inModelUnit))
    {
      theResult = UNITS.DAY;
    }
    // end: should be lookup in template
    return theResult;
  }

  public static <T extends PrimitiveNumber> Time getQuantity(T inValue, UnitsTime inModelUnit)
    throws EnumerationException
  {
    // "Positive" variants are listed before "NonNegative" variants because they extend the "NonNegative" variants and
    // would otherwise be ignored because the "NonNegative" test would be chosen for both "Positive" and "NonNegative"
    // types.  We also process Long before Integer for the same reason since the static analyser also found this led
    // to the Long codepath being picked for the Integer type, though it's not as clear why that is happening.  The
    // current order passes static analysis with all codepaths being correctly visited.
    if (inValue instanceof PercentFraction) {
      PercentFraction typedValue = (PercentFraction) inValue;
      return new ome.units.quantity.Time(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof PositiveFloat) {
      PositiveFloat typedValue = (PositiveFloat) inValue;
      return new ome.units.quantity.Time(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof NonNegativeFloat) {
      NonNegativeFloat typedValue = (NonNegativeFloat) inValue;
      return new ome.units.quantity.Time(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof PositiveLong) {
      PositiveLong typedValue = (PositiveLong) inValue;
      return new ome.units.quantity.Time(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof NonNegativeLong) {
      NonNegativeLong typedValue = (NonNegativeLong) inValue;
      return new ome.units.quantity.Time(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof PositiveInteger) {
      PositiveInteger typedValue = (PositiveInteger) inValue;
      return new ome.units.quantity.Time(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    if (inValue instanceof NonNegativeInteger) {
      NonNegativeInteger typedValue = (NonNegativeInteger) inValue;
      return new ome.units.quantity.Time(typedValue.getValue(), getBaseUnit(inModelUnit));
    }
    LOGGER.warn("Unknown type '{}' cannot be used to create a 'Time' quantity",
      inValue.getClass().getName());
    throw new EnumerationException("UnitsTimeEnumHandler: type '"
      + inValue.getClass().getName() + "' cannot be used to create a quantity");
  }

  public static <T extends Number> Time getQuantity(T inValue, UnitsTime inModelUnit)
    throws EnumerationException
  {
    if (inValue instanceof Double) {
      Double doubleValue = (Double) inValue;
      return new ome.units.quantity.Time(doubleValue, getBaseUnit(inModelUnit));
    }
    if (inValue instanceof Integer) {
      Integer intValue = (Integer) inValue;
      return new ome.units.quantity.Time(intValue, getBaseUnit(inModelUnit));
    }
    LOGGER.warn("Unknown type '{}' cannot be used to create a 'Time' quantity",
      inValue.getClass().getName());
    throw new EnumerationException("UnitsTimeEnumHandler: type '"
      + inValue.getClass().getName() + "' cannot be used to create a quantity");
  }

  /* @see IEnumerationHandler#getEntity() */
  public Class<? extends Enumeration> getEntity() {
    return UnitsTime.class;
  }

}
